/* cal [1.0.7] - 01-Mar-93 11:24                                 John Layman */

/*****************************************************************************\
*                                                                             *
* Synopsis: cal [[mo ]yr]                                                     *
*    where: yr must include the century (e.g. 1987, as opposed to 87).        *
*                                                                             *
* This program produces a calendar for the specified year or month.  When the *
* command line contains no parameters, the calendar for the present month is  *
* printed, flanked by the previous and following months.  The calendar used   *
* is that for England and her colonies.  The year is always considered to     *
* begin with January, even though this is historically inaccurate.            *
\*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <dos.h>
#include "local.h"

static char progid[9] = "cal";

static int cal[12][6][7];	/* 12 months, 6 weeks, 7 days */

static int days[] = { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31, };

static char *month[] = { "January", "February", "March", "April", "May", "June",
		"July", "August", "September", "October", "November", "December", };

static char hdr[] = { " S  M  T  W  T  F  S" };

enum month { Jan, Feb, Mar, Apr, May, Jun, Jul, Aug, Sep, Oct, Nov, Dec };

enum day { Sun, Mon, Tue, Wed, Thu, Fri, Sat };

static void bldcal(int, int);
static int jan1(int);
static void prtyr(int);
static void prtqtr(char *, char *, char *);
static void prtmo(int, int);
static void prtda(int);
static char *center(char *, int);

main(argc, argv)
	int argc;
	char *argv[];
{
	int yr, mo, y[3], m[3], wk, dow, i;
	struct date curr;

	argv0(progid);

	switch (argc)
	{
		case 1:
			/* Default to system date when no arguments are present. */
			getdate(&curr);
			mo = curr.da_mon-1;
			yr = curr.da_year;
			break;
		case 2:
			/* Construe single argument as year. */
			mo = Jan;
			yr = atoi(argv[1]);
			break;
		case 3:
			/* Construe arguments as month year. */
			mo = atoi(argv[1])-1;
			yr = atoi(argv[2]);
			break;
		default:
			fprintf(stderr, "usage: %s [[month ]year]\n", progid);
			exit(1);
	}
	if (mo < Jan || mo > Dec || yr < 1 || yr > 9999)
	{
		fprintf(stderr, "%s: Invalid parameter.\n", progid);
		exit(1);
	}

	switch (argc)
	{
		case 1:
			/* Print calendar for trimester. */
			if (mo == Jan)
			{
				m[0] = Dec; y[0] = yr-1;
			}
			else
			{
				m[0] = mo-1; y[0] = yr;
			}
			m[1] = mo; y[1] = yr;
			if (mo == Dec)
			{
				m[2] = Jan; y[2] = yr+1;
			}
			else
			{
				m[2] = mo+1; y[2] = yr;
			}
			for (i = 0; i <= 2; i++)
				bldcal(m[i], y[i]);
			prtyr(yr);
			prtqtr(month[m[0]], month[m[1]], month[m[2]]);
			for (wk = 0; wk <= 5; wk++)
			{
				for (i = 0; i <= 2; i++)
				{
					for (dow = Sun; dow <= Sat; dow++)
						prtda(cal[m[i]][wk][dow]);
					printf("%s", (char*)((i < 2) ? "  " : "\n"));
				}
			}
			break;
		case 2:
			/* Print calendar for entire year. */
			for (mo = Jan; mo <= Dec; mo++)
				bldcal(mo, yr);
			prtyr(yr);
			for (mo = Jan; mo < Dec; mo += 3)
			{
				prtqtr(month[mo], month[mo+1], month[mo+2]);
				for (wk = 0; wk <= 5; wk++)
				{
					for (i = 0; i <= 2; i++)
					{
						for (dow = Sun; dow <= Sat; dow++)
							prtda(cal[mo+i][wk][dow]);
						printf("%s", (char*)((i < 2) ? "  " : "\n"));
					}
				}
			}
			break;
		case 3:
			/* Print calendar for month. */
			bldcal(mo, yr);
			prtmo(mo, yr);
			for (wk = 0; wk <= 5; wk++)
			{
				for (dow = Sun; dow <= Sat; dow++)
					prtda(cal[mo][wk][dow]);
				printf("\n");
			}
			break;
	}
}


/*****************************************************************************\
* Build calendar array for the given month and year.                          *
\*****************************************************************************/
void bldcal(mo, yr)
	int mo, yr;
{
	int i, wk, dow;

	wk = 0;
	dow = jan1(yr);
	switch ((jan1(yr+1)+7-dow)%7)
	{
		case 1: /* non-leap year */
			days[Feb] = 28;	days[Sep] = 30;
			break;
		case 2: /* leap year */
			days[Feb] = 29; days[Sep] = 30;
			break;
		default: /* 1752 */
			days[Feb] = 29; days[Sep] = 19;
	}
	for (i = Jan; i < mo; i++)
		dow += days[i];
	dow %= 7;
	for (i = 1; i <= days[mo]; i++)
	{
		cal[mo][wk][dow] = i;
		if (yr == 1752 && mo == Sep && i > 2)
			cal[mo][wk][dow] += 11;
		if (dow < Sat)
			dow++;
		else
		{
			dow = Sun;
			wk++;
		}
	}
}


/*****************************************************************************\
* Return the day of week (0..6) for January 1st of the given year.            *
\*****************************************************************************/
int jan1(yr)
	int yr;
{
	int da;

	da = 4+yr+(yr+3)/4;
	if (yr > 1800)
	{
		da -= (yr-1701)/100;
		da += (yr-1601)/400;
	}
	if (yr > 1752)
		da += 3;

	return(da%7);
}


/*****************************************************************************\
* Print calendar year heading.                                                *
\*****************************************************************************/
void prtyr(yr)
	int yr;
{
	int i;
	char str[66];

	sprintf(str, "%u", yr);
	i = strlen(str);
	str[(i*2)-1] = str[i];
	while (--i > 0)
	{
		str[i*2] = str[i]; str[(i*2)-1] = ' ';
	}
	printf("%s\n\n", center(str, sizeof(str)));
}


/*****************************************************************************\
* Print month headings for one quarter.                                       *
\*****************************************************************************/
void prtqtr(m1, m2, m3)
	char *m1, *m2, *m3;
{
	printf("         %.3s           ", m1);
	printf("         %.3s           ", m2);
	printf("         %.3s\n",          m3);
	printf("%s   %s   %s\n", hdr, hdr, hdr);
}


/*****************************************************************************\
* Print month heading for single month.                                       *
\*****************************************************************************/
void prtmo(mo, yr)
	int mo, yr;
{
	char str[20];

	sprintf(str, "%s %u", month[mo], yr);
	printf("\n%s\n", center(str, sizeof(str)));
	printf("%s\n", hdr);
}


/*****************************************************************************\
* Print formatted calendar day.                                               *
\*****************************************************************************/
void prtda(da)
	int da;
{
	if (da == 0)
		printf("   ");
	else
		printf("%2d ",da);
}


/*****************************************************************************\
* Center text within string (insert leading spaces only).                     *
\*****************************************************************************/
char *center(str, len)
	char *str;
	int len;
{
	char *s, *d;
	int i, l, o;

	l = strlen(str);
	o = ((len-l)/2); o += o%2;
	s = str+l;
	d = s+o;
	for (i = o+l+1; i > 0; i--)
		if (i > o)
			*d-- = *s--;
		else
			*d-- = ' ';
	return str;
}
