/* calendar [1.0.7] - 01-Mar-93 11:31                            John Layman */

/*****************************************************************************\
*                                                                             *
* Synopsis: calendar [span]                                                   *
*                                                                             *
* This program reads the file named "calendar" in the current directory, and  *
* copies to stdout any lines which fall within a date window commencing with  *
* the current date.  The span of this window defaults to one day (i.e. today  *
* and tomorrow), but an integer command line argument may be given to specify *
* a longer span.  The window is slightly elastic; it is lengthened to include *
* the following Monday whenever it ends on a Saturday or Sunday.              *
*                                                                             *
* Beginning with version 1.0.5, the root directory is checked for a calendar  *
* file when none is present in the current directory.                         *
*                                                                             *
* Each calendar entry is limited to a single line.  The line must begin with  *
* one of the following date formats:                                          *
*                                                                             *
*                                  mm/dd/yyyy                                 *
*                                  mm/dd/yy                                   *
*                                  mm/dd                                      *
*                                  dd                                         *
*                                                                             *
* where hyphens or whitespace may also be used as separators.  When the month *
* and/or year is omitted, the forthcoming corresponding date is implied.      *
*                                                                             *
* Take care, when omitting date components, to format the line such that the  *
* date is clearly distinct from any subsequent number.  Note that the entry:  *
*                                                                             *
*                   6/15 - 12:30 - meet Alan for lunch at Met Grill           *
*                                                                             *
* would be interpreted as June 15, 1912, whereas:                             *
*                                                                             *
*                   6/15 @12:30 - meet Alan for lunch at Met Grill            *
*                                                                             *
* would be interpreted as June 15th, next.                                    *
\*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <dos.h>
#include "local.h"

#define MAXLEN	256

static char progid[9] = "calendar";

static int dow(long day);
static long day(int mo, int da, int yr);

main(argc, argv)
	int argc;
	char *argv[];
{
	FILE *fp;
	int currmo, currda, curryr, mo, da, yr;
	long from, thru, d;
	char str[MAXLEN];
	enum days { Sun, Mon, Tue, Wed, Thu, Fri, Sat };
	struct date curr;

	argv0(progid);

	if (argc > 2 || ((argc == 2) && (atoi(argv[1]) == 0)))
	{
		fprintf(stderr, "usage: %s [range]\n", progid);
		exit(1);
	}
	if ((fp = fopen("calendar", "r")) == NULL)
	{
		if ((fp = fopen("\\calendar", "r")) == NULL)
		{
			fprintf(stderr, "%s: Cannot open file: calendar.\n", progid);
			exit(1);
		}
	}

	/* Calculate date range to be selected = today..today+n. */
	getdate(&curr);
	currmo = curr.da_mon;
	currda = curr.da_day;
	curryr = curr.da_year;
	from = day(currmo, currda, curryr);
	thru = from+((argc == 1) ? 1 : atoi(argv[1]));
	while (dow(thru) < Mon || dow(thru) > Fri) thru++; /* Adjust for weekend */

	/* For each line in calendar which begins with an identifiable date, */
	/* print those lines which fall within the selected range.           */
	while ((int)(fgets(str, sizeof(str), fp)) != NULL)
	{
		switch (sscanf(str, "%2d%*[-/]%2d%*[-/]%4d ", &mo, &da, &yr))
		{
			case 1:
				da = mo; mo = currmo; yr = curryr;
				if (da < currda) mo++;
				if (mo > 12) { mo = 1; yr++; }
				break;
			case 2:
				yr = curryr;
				if (mo < currmo) yr++;
				break;
			case 3:
				if (yr < 100) yr += 1900;
				break;
			default:
				continue;
		}
		d = day(mo, da, yr);
		if (d >= from && d <= thru)
			fputs(str, stdout);
	}
}


/*****************************************************************************\
* Calculate day number.                                                       *
\*****************************************************************************/
long day(mo, da, yr)
	int mo, da, yr;
{
	yr -= (mo < 3);
	return (1461*(long)yr)/4+(306*((mo+9)%12+4))/10+da-(yr/100)+(yr/400);
}


/*****************************************************************************\
* Calculate day of week.                                                      *
\*****************************************************************************/
int dow(day)
	long day;
{
	/* Calculate day of week (0..6 = Sun..Sat) for given day number. */
	return (day-1)%7;
}
